package DrawHills;


import java.awt.*;
import java.lang.Math;

import DrawTexts.*;
//import Globals.GS; //To enable console.




//*******************************************************
/**<pre>

 This is a basic super class for all 3D-paint procedures.
 @author Konstantin Kirillov
 @version 1.0
 Copyright (C) 2007 Konstantin Kirillov. 

 This class must have onw Scratch Pads, so it is
 dependent on them.

</pre>*/
//*******************************************************

public class SingleHill {


//=============================================================
// Contol
//-------------------------------------------------------------
public static boolean           flagTrianglesVsSquares = false;
/** Set this for debug mode */  protected boolean debug = true;
/** Labels: Axis, ...       */  protected boolean displayAxes;
//=============================================================


//=====================================================
// Numeric algorithms parameters.
//-----------------------------------------------------
//3d perspective:
protected   int safeModule = 10;   
protected   int safeScale = 1<<safeModule;
protected int tangentOfVision = 1;  //do2 Do in future.
//=====================================================



//================================================================
// Positions in Flat Valley
//----------------------------------------------------------------
/** <pre>
* pos - Pos of Hill in Flat Valley. Iww, in Flat Local Scene.
* They are set at function definition.
* They are not used in internal paint procedures.
* They are used only to define hOO.
</pre> */
protected int posX=0, posY=0, posZ=0; 
/**
 * obs - Pos of Obs in Flat Valley.
 *       Supplied in PaintHill proc.
*/
protected int obsX, obsY, obsZ;       


/**
 *  hOO - Pos in Observer basis respectively to Obs.
*/
protected int hOOx, hOOy, hOOz;      

/** Values not used when |angle| < .01 */
protected int altitudeCos, altitudeSin;   
protected double oAltitudeViewAngle = 0.;


//----------------------------------------------------------------
// Positions in Flat Valley
//================================================================

//================================================================
/** Functions templates:
    There are two ways to use functions.
    Supply this funN for this constants for predefined functions.
    Or, override Fun method when deriving a class.
*/
public static final int funTerrain               =  0; 
public static final int funFlippedParabola       =  1;
public static final int funY2mX2                 =  3;
public static final int funGaussBell             =  5;
public static final int funPolinomials           =  8;
public static final int funUpperSphere           =  9;
public static final int funLowSphere             = 10;

public int funN;    //Selected function.

protected double shieldRadius =  250.; //Protects Hill from entering.
                                       //Defined in function definition.

//================================================================



//==============================================================
// Geometry, working variables and parameters.
//--------------------------------------------------------------

protected int Xmin, Xmax, Xstep;
protected int Ymin, Ymax, Ystep;
public    int YMaxMin;                 //Approximate distance.

//>>>>>>> Speed up rotation by shifts: >>>>>>>>>
protected int aziRotationShift = 7;
//Must be equal 2^aziRotationShift:
protected int RotationModula   = 128; 
//do2 values are redundant?:
protected int SinRotationAngle = 0;             
protected int CosRotationAngle = RotationModula;
//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


protected int RotatedYAuxilairy = 0;   //ugly global par. because
                                       //I did not know how to return array.

//Eye's Screen:
protected int center_x;
protected int center_y; 
protected int ScreenXL, ScreenYL;

protected int CurrScreenX, CurrScreenY;
protected int CurrFactor;


//---------------------------------------------------------
// Meshes and their parameters
// Run-time Sceleton = Grid inside function's domain.
// Fixed when observer does not move and function is static.
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

/** Remember: Xsize * Yrange < ReservedDomain
*/
protected int ReservedDomain = 10000;
                             
protected int Xsize =40;     // Xsize = number of knots on an axis x.
protected int Yrange=40;     // Yrange - the same for axis y.                          


protected int firstMeshDrawIx=0;
protected int lastMeshDrawIx =-1;

protected int screenMeshX [][] = new int [ReservedDomain][3];
protected int screenMeshZ [][] = new int [ReservedDomain][3];

/** This is a mesh: "patches" on function's surface:
    See triangles ABC and ACD below in ContructHill method.
    screenMesh2X - for second triangle.
*/
protected int screenMesh2X [][] = new int [ReservedDomain][3]; 

/** For second triangle. */
protected int screenMesh2Z [][] = new int [ReservedDomain][3]; 


//This are normal vectors to funciton's surface:
protected int nX[] = new int [ReservedDomain];
protected int nY[] = new int [ReservedDomain];
protected int nZ[] = new int [ReservedDomain];

protected int n2X[] = new int [ReservedDomain];
protected int n2Y[] = new int [ReservedDomain];
protected int n2Z[] = new int [ReservedDomain];
//---------------------------------------------------------


//--------------------------------------------------------------
// Geometry, working variables and parameters.
//==============================================================


//============================================
// Auxilary meshes on screen and ?surface
//--------------------------------------------
   //shadow constructions:
   //shadow front:
//first [] - 0 - x;  1 - z:
protected   int [][] screen_current   = new int[2][Xsize];  
//first [] - 0 - rHO; 1 - RZ:
protected   int [][] Real_current     = new int[2][Xsize];  

//shadow feet: ?boudary of filled area: low-computer-display-boundary?
protected   int [][] Real_previous   = new int[2][Xsize];  
protected   int [][] screen_upper    = new int[2][Xsize];  

//former (one step back along Y) value;
//"upper" is not a good term: previous Y's
//value is higher on display if X=Z=0.

//============================================   



//==============================================================
//Lights:
//--------------------------------------------------------------

public   Color BackgroundColor = new Color(0);
//public Color BackgroundColor = new Color(255<<16|255<<8|255);

public    Color OwnColor=new Color(255<<8|255);
protected Color hillAxisColor = new Color( 100<<16 | 255<<8 | 100);

//Values in case if MoonHalfPeriod = 0.
protected int phase_sub =  0; 
protected int phase_sig = -1;

protected int MoonZ                   = 100;
protected int MoonHorizontalRadius    = 400;
protected int MoonHalfPeriod          = 50; //in frames.
protected int MoonXStart              = 0;


//Current values:
protected int lightZ,lightY,lightX;  //In Hills basis.
protected int REventX, REventY;      //In Obs. basis.
protected int radius;                //To Hill center in Hill's reference.
//--------------------------------------------------------------
//Lights:
//==============================================================








//============================================
// Graphics and Text
//--------------------------------------------
protected Graphics g;
/** Labels: Text area ...
    spOwn is coinsided 
    with own graphics screen.       */  protected SPad    sp, spOwn; 
//============================================


//================================================================
//  Misc.
//----------------------------------------------------------------
/** Analog of hills internal time:
    Number of calls to hill:
    Can be used if external synch 
    is not supplied.                */  protected int     frameNumber = 1; 
/** Auxilairy                       */  private   double  PI = Math.PI;
//----------------------------------------------------------------
//  Misc.
//================================================================






//================================================================
//  S u p p l y   a n d   r e l e a s e   r e s o u r c e s
//----------------------------------------------------------------
/** Make sure that this is executed after contructor
    after own Scratch Pad Graphics is supplied. */
public void supplyGraphics() {
       //GS.con("Supplying Graphics to the Hill " + this );
       this.g = spOwn.g;
}
public void releaseGraphics() {
       g = null;
}
//================================================================







//============================================
public SingleHill() {}

public SingleHill(  
              SPad sp,
              SPad spOwn,
              int ScreenXL,    
              int ScreenYL,
              int center_x,
              int center_y,
              int funN
   ) {


   this.sp              = sp;
   this.spOwn           = spOwn;
   this.ScreenXL        = ScreenXL;
   this.ScreenYL        = ScreenYL;
   
   this.center_x = center_x;
   this.center_y = center_y;

   this.funN = funN;
   


//============================================
//  Functions
//--------------------------------------------

if( funN == funFlippedParabola | 
    funN == funGaussBell ) {
    posX =100;
    posY =500;
    posZ =0;
    Xmin =-200;
    Xmax =200;
    Ymin =-200;
    Ymax =200;
    shieldRadius = 250.;

}

if( funN == funPolinomials ) {
    posX = -200;
    posY =800;
    Xmin=-400;
    Xmax=400;
    Ymin=-400;
    Ymax=400;
    shieldRadius = 0.;
    hillAxisColor = new Color( 200<<16 ); 
} 


if( funN == funUpperSphere ) {

    posX = 0;
    posY = 0;
    posZ = 0;
    Xmin =-100;
    Xmax = 100;
    Ymin =-100;
    Ymax = 100;

    MoonHalfPeriod          = 0;
    lightX                  = 50;
    lightY                  = -200;
    lightZ                  = 50;

}
//--------------------------------------------
//  Functions
//============================================




//============================================
//  Preparations
//--------------------------------------------
   Xstep  = (Xmax-Xmin+1)/Xsize;
   Ystep  = (Ymax-Ymin+1)/Yrange;
   YMaxMin = Math.max( 
               Math.max( Math.abs(Ymax),Math.abs(Ymin) ),
               Math.max( Math.abs(Xmax),Math.abs(Xmin) )
             );
//============================================

}


 


/**
  * <pre>
  * This is main interface to paint procedures, main gate.
  *
  * @param obsX  Observer's X-position in Flat Valley.
  *
  * @param ExternalFrameNumber synchronize with external state frame number
  * if < 0 then internal frame numeration is used.
  *
  * @param obsY1 projection of n on axis X = Cos(FI)
  * @param obsY2 projection of n on axis Y = Sin(FI)
  * @param obsY3 projection of n on axis Z = Cos(PSY) - not used
  *
  * If observer turns the head clockwise, then FI > 0
  * The Hill turns couterclockwise, so x'=xCos(FI)-ySin(FI)
  * Here: obsY1=nx=Cos(FI), ny=obsY2=Sin(FI), nz=obsY3=Cos(PSY).
  * vector n=observer's direction after observer's turn.
  *
  * (Note: if to measure obs turn respect to axis X, FI|X, then
  *  FI|Y = FI|X - PI/2.)
  *
  * @param fillScrBackgroundBeforeDraw
  * </pre>
  *
*/
public void paintHill(int obsX, int obsY, int obsZ, 
                      double obsY1, double obsY2, double obsY3, 
                      int ExternalFrameNumber, 
                      boolean fillScrBackgroundBeforeDraw, 
                      boolean dispalyMoon,
                      boolean displayAxes,
                      double oAltitudeViewAngle
                      ) {

       if(ExternalFrameNumber > -1) frameNumber=ExternalFrameNumber;
        
       int hOFx,  hOFy;

       this.obsX = obsX;
       this.obsY = obsY;
       this.obsZ = obsZ;

       hOFx=posX-obsX;
       hOFy=posY-obsY;

       CosRotationAngle = (int) (RotationModula *   obsY2 );
       SinRotationAngle = (int) (RotationModula *   obsY1 );

       if( Math.abs( oAltitudeViewAngle ) > .009 ) {
           this.oAltitudeViewAngle = oAltitudeViewAngle;
           altitudeCos      = (int) (RotationModula * Math.cos(oAltitudeViewAngle));
           altitudeSin      = (int) (RotationModula * Math.sin(oAltitudeViewAngle));
       }else{
           this.oAltitudeViewAngle = 0.;
       }

       hOOx = (int) (hOFx * obsY2   + hOFy * (-obsY1));
       hOOy = (int) (hOFx * obsY1   + hOFy *   obsY2 );
       hOOz = posZ-obsZ;

       paintHillInternally( true,
                            fillScrBackgroundBeforeDraw, 
                            dispalyMoon,
                            displayAxes
                          );
       if(ExternalFrameNumber < 0) frameNumber+=1;

}



protected void paintHillInternally(
                      boolean recalculate_function,
                      boolean fillScrBackgroundBeforeDraw, 
                      boolean dispalyMoon,
                      boolean displayAxes
                      ) {

       PositionMoon();
       g.setColor(BackgroundColor);
       if( fillScrBackgroundBeforeDraw ) {
           g.fillRect(0,0,ScreenXL,ScreenYL);
       }

       if( dispalyMoon ) {
           if(REventY>0) { 
              DisplayMoon(); //Moon behind the hill. 
              //sp.wr("Behind"); //debug
           }
       }

       if(recalculate_function) {
          ConstructHill(); //This PaintFunction(true); will be
                            //inside of Construct Hills.
       }
       PaintFunction();

       if( displayAxes ) {
           DrawCoordinateAxis();
           DrawAxisToHill();
       }

       if( dispalyMoon ) {
           if(REventY<0) DisplayMoon(); //Moon before the hill.
       }

       PaintAuxSigns();

}



//PPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP
//  ConstructHill
//======================================================================================================


protected void ConstructHill() {

int iii;     //Used for index of arrays.
int Z=0;
int factor = 1;    //Give compiler something to eat ...
int frontDistance; 
int centerz;

int x=0,z=0;       // x,z   - are aSS, S - Screen of Computer Display.
int xI=0,X=0,Y=0;  // X,Y,Z - are aHH, a - funcitons argument.
int xHO=0, yHO=0;  


//Matter of algorithm: "draw farer regions first."
//if Ynext > Yformer then = 1; otherwise -1;
int DirectionRelativeToPositive=-1;	

centerz=center_y;

//Loop along axis which is perpendicular to observers front plane:
iii = -1;
firstMeshDrawIx = iii+1;
yloop:
for(int Ynorm=0; Ynorm<Yrange; Ynorm++ ) { //LOOP2(via Ynorm)

     //------------------------------------------
     //Set what direction coordinate Y will move:
     //from viewer or to viewer:

     yHO = Ymax - (Ynorm+1)*Ystep;   //For Terrain, Ymax and Ymin are in respect to Oblserver.
                                     //For other functions - in respect to Hill.
     if( funN == funTerrain ) yHO -= hOOy;
     frontDistance = yHO+hOOy;       //Vector from Observer to a.

     if(funN == funTerrain) { //Insert subhill picture here if any.
        InsertSubHill(iii, frontDistance);
     }  

        //Observer entered hill.
        //Hill still will be painted, but partially, only for
        //front planes before the observer:
        //Too arbitrary criteria ...

     if( oAltitudeViewAngle == 0. ) {
         if(frontDistance < Ystep) break;
         factor = safeScale * ScreenXL /frontDistance;
     }

     int XminAdopted;

     Xstep = (Xmax - Xmin + 1) / Xsize;
     int wi = tangentOfVision * frontDistance / Xsize ;
     if(wi < 2 ) wi = 2;
     if(wi < Xstep ) {
        //Xstep = wi;  //do2. Should be done for y also. Perhaps looks wierd.
        XminAdopted = - tangentOfVision * frontDistance / 2 - 3;  //3 - for safety.
     } else {
        XminAdopted = Xmin;
     }

  int xIsOutOfScreen = 0;
  //////////////////////////////////////////////////////////////////////////////////////
  // xI-cycle. LOOP3. Along horizontal axis which is parallel to observer's front plain.
  //====================================================================================
  for(xI=0; xI<Xsize; xI++) {        //LOOP3 
                                     //xI-cycle, xI- Index of points along real axis X.
      xHO=xI*Xstep+XminAdopted;      
      if( funN == funTerrain ) xHO -= hOOx;               

      X = RotateArgToFunctDomainBasisX( xHO,yHO );  
      Y = RotatedYAuxilairy;
      Z = Fun( funN,X,Y );


      if( oAltitudeViewAngle == 0. ) {
          x = center_x + (  (factor*(xHO+hOOx)) >> safeModule  );
          z = center_y - (  (factor*(Z  +hOOz)) >> safeModule  ); 
      }else{
          int altiY0 = yHO+hOOy;
          int altiZ0 = Z  +hOOz;
          int altiZ = (altitudeCos * altiZ0 - altitudeSin * altiY0)/RotationModula;
          int altiY = (altitudeSin * altiZ0 + altitudeCos * altiY0)/RotationModula;
          if( altiY < Ystep ) break yloop; //Assume edge of picture reached.
          int wFactor = safeScale * ScreenXL /altiY;
          //Flat coord on the screen = x
          x = center_x + (  (wFactor*    X) >> safeModule  );
          //Vertical coodinate z, is put from the top of computer display:
          z = center_y - (  (wFactor*altiZ) >> safeModule  ); 
      }
      
      if( x>= ScreenXL ) {
          x=ScreenXL-1;
          xIsOutOfScreen += 1;
          if(xIsOutOfScreen>5)break;    //Don't use redundant nodes.
      }
      if( x<0 ) x=0;




      screen_current[0][xI]=x;
      screen_current[1][xI]=z;
      //::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

      //coords in real space:
      Real_current[0][xI]=xHO; 
      Real_current[1][xI]=Z;
      
      //LOOP3 //xI-cycle continued:
      /////////////// this is the first(=starting) line "Ybeginning" //////////////////////
      if( Ynorm == 0) {
          //IF1 inside of (xI-cycle)
          //Y-shadow feet;	   
          screen_upper[0][xI]=screen_current[0][xI];		
	    //KK: seems first value (really = bottom)
	    screen_upper[1][xI]=screen_current[1][xI];
          //continue;
      } else { //Y-shadow body
       
        if(xI>0) { 
                
            //IF2 xI>0
            //Vectors and coodinates are here:
            //  in observer's basis,
            //  are of type (int),
            //  safeModule is not used for them.

            //If need to fix light distribution, put vector 
            //product and other statements here;

            // vector by shift along axis X:
            // like: (-XMin,Z(XMin)-->(-XMin+Xstep),Z(XMin+Xstep))

            /*
                    B'      C'    
                    *-------> c 
                   /..     /
                  / . .   /
                 /  .  . / 
                /   .   / 
               /    .  / \
              >l-------   \
              A'    |      \
                    |       > n = c*l
                    |       
                    o center
                             

               light source: <------------o center


                On screen:

                    B----C First points for this triangle.
                    |   /
                    |  /
                    | /
                    A/
              
            */ 	                    


            //Vector c (corresponds B->C):
            int cx = Xstep;
            //int cy = 0;
            int cz = Z-Real_current[1][xI-1];

            //Vector l by shift along axis Y (corresponds A-->B):
            //int lx = 0;
            int ly =  Ystep * DirectionRelativeToPositive;
            //do1 bug:
            int lz = Real_current[1][xI-1]-Real_previous[1][xI-1];
		 
            //Make vector product, n = [c x l]:
            //int nx = (cy*lz-cz*ly);
            //int ny = (-cx*lz + lx*cz);
            //int nz = (cx*ly-cy*lx);		               
            int nx = -cz*ly;
            int ny = -cx*lz;
            int nz =  cx*ly;		               

            int module = root( nx*nx+ny*ny+nz*nz );
            if(module <= 0) module = 1;


                
            iii +=1;
            nX[iii] = 250*nx/module;
            nY[iii] = 250*ny/module;
            nZ[iii] = 250*nz/module;

            screenMeshX[iii][0]=screen_upper[0][xI-1];   //A
            screenMeshZ[iii][0]=screen_upper[1][xI-1];
	
            screenMeshX[iii][1]=screen_current[0][xI-1]; //B
            screenMeshZ[iii][1]=screen_current[1][xI-1];

            screenMeshX[iii][2]=x;   //C
            screenMeshZ[iii][2]=z;
            //recall:  screen_current[0][xI]=x;
            //         screen_current[1][xI]=z;


          if( flagTrianglesVsSquares ) {

               /*      C Second three points for this triangle.
                      /|
                     / |
                    /  |
                  A/---D
               */		   

            //Vector c (corresponds A->D):
            //cx = Xstep; //already done.
            //cy = 0;
            cz = Real_previous[1][xI]-Real_previous[1][xI-1];

            //Vector l by shift along axis Y (corresponds D-->C):
            //lx = 0;
            //int ly =  Ystep * DirectionRelativeToPositive;
            lz = Z-Real_previous[1][xI];
            //Recall:            Real_current[1][xI]=Z;
		 
            //Make vector product, n = [c x l]:
            //int nx = (cy*lz-cz*ly);
            //int ny = (-cx*lz + lx*cz);
            //int nz = (cx*ly-cy*lx);		               
            nx = -cz*ly;
            ny = -cx*lz;
            nz =  cx*ly;		               

            module = root( nx*nx+ny*ny+nz*nz );
            if(module <= 0) module = 1;

            n2X[iii] = 250*nx/module;
            n2Y[iii] = 250*ny/module;
            n2Z[iii] = 250*nz/module;

         }


            screenMesh2X[iii][0]=screen_upper[0][xI-1];   //A
            screenMesh2Z[iii][0]=screen_upper[1][xI-1];

            screenMesh2X[iii][1]=x;                       //C
            screenMesh2Z[iii][1]=z;
	
            screenMesh2X[iii][2]=screen_upper[0][xI];     //D
            screenMesh2Z[iii][2]=screen_upper[1][xI];

              //xI-cycle. LOOP3. continued;
              //IF1 Y-shadow body continued;
              //IF2 xI>0 continued;

         } //IF2 xI>0  block end		
        }  //IF1 Ynorm == 0 (Y-shadow feet and body block end )
      }    //LOOP3 (xI-"normed" coordinate) end

      for( xI=0; xI<Xsize; xI++) {
	   for(int i=0; i<2; i++) { 
             Real_previous[i][xI] = Real_current[i][xI]; 
	       screen_upper[i][xI]=screen_current[i][xI];	 
         }
      }
    } //LOOP2(Ynorm); seems: function domain loop end;
    lastMeshDrawIx = iii;  //sp.wr( "mesh nodes count= " + meshL );
    if(funN == funTerrain) PaintFunction();

//======================================================================================================
}   //ConstructHill 
//PPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP





//PPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP
//  PaintFunction
//======================================================================

protected void PaintFunction() {

int iii;     //Used for index of arrays.
int intColor;
Color drawColor;

int loRed = 100;
int loBlue = 0;
int loGreen = 0;

for( iii=firstMeshDrawIx; iii<=lastMeshDrawIx; iii++ ) {

     //////////////////////////////////////////////////////////////////////////////////
     //  set color: 
     //================================================================================
     //Here is a part which depends on light vector direction:
     //In line below: every vector is in observer basis:
     //Vector REvent direction to ligh source is directed from Hills center. 
     //Bright surface has N directed to Hills center.
     //This is why sign minus is here:

     intColor = -(   (nX[iii]*REventX + nY[iii]*REventY + nZ[iii]*lightZ)/radius   );

     //See comment: "REvent is the same vector, but in Observer's basis ...
     //Perhaps: use normal this vector in form: 0,-1,0:
     //if( nY[iii] < 0 ) intColor = - intColor;

     if( intColor < 0 )   intColor =0;        
     //intColor = intColor * 2;            //Don't do this. This is a mistake. 
                                           //This variable already uses a full range.
     if( intColor > 255 ) intColor = 255;

     loRed   = OwnColor.getRed()   * intColor / 256;  //  /256 is because nX is normed
                                                      //  to 250.
     loBlue  = OwnColor.getBlue()  * intColor / 256;
     loGreen = OwnColor.getGreen() * intColor / 256;

     drawColor = new Color( loRed<<16|loGreen<<8|loBlue<<0 ); 
     g.setColor(drawColor);
     //================================================================================
     //  set color: 
     //////////////////////////////////////////////////////////////////////////////////

     Polygon pol1 = new Polygon(screenMeshX[iii],screenMeshZ[iii], 3);
     g.fillPolygon( pol1 );

     if( flagTrianglesVsSquares ) {
         intColor = -(   (n2X[iii]*REventX + n2Y[iii]*REventY + n2Z[iii]*lightZ)/radius   );
         if( intColor < 0 )   intColor =0;        
         if( intColor > 255 ) intColor = 255;
         loRed   = OwnColor.getRed()   * intColor / 256;  
         loBlue  = OwnColor.getBlue()  * intColor / 256;
         loGreen = OwnColor.getGreen() * intColor / 256;
         drawColor = new Color( loRed<<16|loGreen<<8|loBlue<<0 ); 
         g.setColor(drawColor);
     }

     Polygon pol2 = new Polygon(screenMesh2X[iii],screenMesh2Z[iii], 3);
     g.fillPolygon( pol2 );

}
     //ms(" funN=" + funN + " Member=" + iXinHillsFamily + 
     //     " MeshDrawIx" + firstMeshDrawIx + " " + lastMeshDrawIx);     
     //spOwn.g.setColor(new Color( 255<<16 | 255 <<8 ));
     //spOwn.g.drawString( "Last word Message" + " funN=" + funN + " Member=" + iXinHillsFamily, 100, 355);
     //spOwn.g.drawString( "Last word Message" + " funN=" + funN + " Member=" + iXinHillsFamily, 100, 375);


//=====================================================================================
}	// ef function PaintFunction
//PPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP





//====================================
protected void DrawCoordinateAxis() {
     int length = (Xmax + Ymax) / 3;
     //Axis X:
     DrawLegendedLine( 0,0,0,   length,0,0,   hillAxisColor, "x");
     //Axis Y:
     DrawLegendedLine( 0,0,0,   0,length,0,   hillAxisColor, "y");
     //Axis Z: (Try length. Zmax is unknown.)
     DrawLegendedLine( 0,0,0,   0,0,length,   hillAxisColor, "z");
}
//====================================


protected void DrawAxisToHill() {
    ProjectObsSpaceLineToScr( 0,20,0, hOOx, hOOy, hOOz,
                              50, 50, 255, "h");
}



//Wrapper, see input parameters in ..ToS function:
public void ProjectObsSpaceLineToScr( int X1, int Y1, int Z1,  int X2, int Y2, int Z2, 
                                      int Col1, int Col2, int Col3, String LTitle)     {
            ProjectObsSpaceLineToS( X1,Y1,Z1,   X2,Y2,Z2, 
                                    new Color( (Col1<<16) | (Col2<<8) | Col3 ), 
                                    LTitle);
}
//INPUT int X1, int Y1, int Z1,  int X2, int Y2, int Z2 - line 
//      in obs reference and in obs basis.
public void ProjectObsSpaceLineToS( int X1, int Y1, int Z1,  int X2, int Y2, int Z2, 
                                    Color c, String LTitle)     {

      int S1X = ObsToSX(X1, Y1, Z1);
      int S1Y = CurrScreenY;

      int S2X = ObsToSX(X2, Y2, Z2);
      int S2Y = CurrScreenY;

      //sp.wrc("X1="  + X1  + " Y1="    + Y1 + " Z1=" + Z1);
      //sp.wr ("S2X=" + S2X + "  S2Y=" + S2Y);

      if(S1X < 0 | S2X < 0 ) {
         //if(debug) g.drawString( "Negative Line", 0, 0);
         return;  
      }

      g.setColor(c);
      g.drawLine( S1X, S1Y, S2X, S2Y );
      g.drawString( LTitle,  S2X+3,  S2Y+3);
          
}







//do2: slow?:
public void DrawLegendedLine( int X1, int Y1, int Z1,  int X2, int Y2, int Z2, int Col1, int Col2, int Col3, String LTitle) {
       DrawLegendedLine( X1, Y1, Z1,  X2, Y2, Z2,   new Color( (Col1<<16) | (Col2<<8) | Col3 ),    LTitle);
} 


public void DrawLegendedLine( int X1, int Y1, int Z1,  int X2, int Y2, int Z2, Color c, String LTitle) {

      int X1R = XRotated (X1, Y1);
      int Y1R = RotatedYAuxilairy;
      int X2R = XRotated (X2, Y2);
      int Y2R = RotatedYAuxilairy;

      int S1X = FindScreenX(X1R, Y1R, Z1);
      int S1Y = CurrScreenY;
      int S2X = FindScreenX(X2R, Y2R, Z2);
      int S2Y = CurrScreenY;

      if(S1X < 0 | S2X < 0 ) {
         g.drawString( "Negative Line", 0, 0);
         return;  
      }

      g.setColor(c);
      g.drawLine( S1X, S1Y, S2X, S2Y );
      g.drawString( LTitle,  S2X+5,  S2Y+5);
          
}






///////////////////////////////////////////////////////////////////////////////////////
//this two programs must be used in order X then Y strictly;

public int XRotated(int X, int Y) {
 RotatedYAuxilairy = ( SinRotationAngle * X  +  CosRotationAngle * Y) 
                       / RotationModula;
 return (CosRotationAngle * X  -  SinRotationAngle * Y) 
        / RotationModula;
}

public int YRotated() {
 return RotatedYAuxilairy;
}


public int RotateArgToFunctDomainBasisX(int X, int Y) {
 RotatedYAuxilairy = ( CosRotationAngle * Y - SinRotationAngle * X  ) 
                     / RotationModula;
 return (CosRotationAngle * X  +  SinRotationAngle * Y) 
        / RotationModula;
}

public int YRotatedBack() {
 return RotatedYAuxilairy;
}
///////////////////////////////////////////////////////////////////////////////////////       



//X,Y,Z must be already rotated and in coordinates relative to Hill.
public int FindScreenX(int X, int Y, int Z ) {

  int DistanceToXZPlain = hOOy + Y;
  if ( DistanceToXZPlain > 0 ) {
       CurrFactor = safeScale * ScreenXL /DistanceToXZPlain;
       CurrScreenX = center_x+((CurrFactor * (X + hOOx))>>safeModule);
       CurrScreenY = center_y-((CurrFactor * (Z + hOOz))>>safeModule); 
  } else {
       CurrScreenX = -1;
       CurrScreenY = -1;
  }
       return CurrScreenX;
}

public int FindScreenY(int X, int Y, int Z) {
       return  CurrScreenY; 
}


//X,Y,Z must be in coordinates 
//      relative to Observer,
//      in Observer's basis.
//OUTPUT If X,Y,Z are behind the screen, -1 is returned.
//First, run this. Then, to take SY, take CurrScreenY.
public int ObsToSX(int X, int Y, int Z ) {
  if ( Y > 0 ) {
       CurrFactor  = safeScale * ScreenXL /Y;
       CurrScreenX = center_x+((CurrFactor * X)>>safeModule);
       CurrScreenY = center_y-((CurrFactor * Z)>>safeModule); 
  } else {
       CurrScreenX = -1;
       CurrScreenY = -1;
  }
       return CurrScreenX;
}





//MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON 

//If MoonXStart < HalfPeriod, then phase starts from 0:
protected void FindMoonPhase(int HalfPeriod) {
     if( HalfPeriod != 0 ) { //Moving Moon
         int phase     = frameNumber + MoonXStart;      //Not used?
             phase_sub = phase % HalfPeriod;            //Subphase - inside period.
         int phase_tot = (phase / HalfPeriod) % 2;      //Global phase: 0 or 1.
             phase_sig = phase_tot * 2 - 1;             //Global phase with sign: -1 or 1;
     } 
}


//Input:  MoonHalfPeriod ==0 => Moon position is fixed and
//                              defined by lightX,lightY,lightZ.
//Output: lightX,lightY,lightZ is vector from Hill's center to
//        the moon in Hill's basis.
//        REvent is the same vector, but in Observer's basis.
protected void PositionMoon() {

     if( MoonHalfPeriod != 0 ) { //Moving Moon
                          

/*       
         int phase     = frameNumber + MoonXStart;      //Not used?
             phase_sub = phase % MoonHalfPeriod;        //Subphase - inside period.

         //If MoonXStart < MoonHalfPeriod, then phase starts from 0:
         int phase_tot = (phase / MoonHalfPeriod) % 2;  //Global phase: 0 or 1.
             phase_sig = phase_tot * 2 - 1;             //Global phase with sign: -1 or 1;
*/

         FindMoonPhase(MoonHalfPeriod);
         //+1 to secure against incorrect rounding:
         lightX = phase_sig * ( MoonHalfPeriod - phase_sub * 2 ) * MoonHorizontalRadius / (MoonHalfPeriod + 1);
         lightY = phase_sig * root(MoonHorizontalRadius * MoonHorizontalRadius - lightX * lightX);
         lightZ = MoonZ;

     }   // Else Moon position is defined by lightX, lightY, lightZ
   
     //Redundant?: To avoid loss of accuracy:
     radius = root(lightY*lightY + lightZ*lightZ + lightX*lightX);

     //Moon in Observer's basis:            
     REventX = XRotated (lightX, lightY);
     REventY = YRotated ();

}



//MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM

protected void DisplayMoon() {

//======================================================================================
 
    int x = FindScreenX(REventX, REventY, lightZ);
    if( x<0 ) return;    //x=0;
    int z = FindScreenY(REventX, REventY, lightZ);
    if( z<0 ) return;    //z=0;

    int oval_stage = 3;  //((frameNumber/10 + 2) % 3)+2;
    int oval_size  = (20 * CurrFactor)>>safeModule;
    if( oval_size < 2 ) oval_size = 2;

    int color_factor  = 255/oval_size;
    int color_factor2 = 125/oval_size;

    Color oval_color = new Color(0);

    if( z+5<ScreenYL && x+5<ScreenXL ) {

        for( int int_oval=1; int_oval<=oval_size; int_oval++ ) {
         
           //int_oval --> => brightness <--? size --> shift x -->
           //Change Event color when it goes to the "past":
           int DoubledOvalColor = int_oval*color_factor * 2;
           if (DoubledOvalColor > 255 ) DoubledOvalColor = 255;

           if(lightY > 0 ) {
              if(lightX < lightY) {               
                 oval_color = new Color(   ((int_oval*color_factor   )  <<16  ) |						  
                                           ((DoubledOvalColor        )  << 8  ) |	
		                               ( int_oval*color_factor   )          );
              } else {  
                oval_color = new Color(   ((int_oval*color_factor   )  <<16  ) |						  
	                                    ((int_oval*color_factor   )  << 8  ) |	
                                          ( DoubledOvalColor )                 );
              }


           } else {  
                oval_color = new Color(   ((DoubledOvalColor        )  <<16  ) |						  
	                                    ((int_oval*color_factor   )  << 8  ) |	
                                          ( int_oval*color_factor   )          );
           }

           g.setColor(oval_color);
           g.fillOval(x+int_oval/oval_stage, z+int_oval/oval_stage, 
                     (oval_size+1-int_oval),(oval_size+1-int_oval));

       }
    }
}
//MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON MOON 




public void ms( String s ) {
     spOwn.wr(s);
}

protected void PaintAuxSigns() {
  //For example:
  //g.drawString("Hill Stage =" + frameNumber, 5, 5);
}





//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
//   F U N C T I O N S   S E L E C T E D   B Y   F U N C T.  N U M B E R
//---------------------------------------------------------------------------
public int Fun( int funNumb, int X, int Y ) {

   int Z=0;

   switch (funNumb) {
   /*
   case funTerrain:
        Z = Terrain( (double) X, (double) Y );
        break;
   */
   case funGaussBell:
        //good: Z =  (int)  1000000 /(Y*Y + X*X + 5000) - 20;
        Z = 800000 /(Y*Y + X*X + 5000) - 20;
        if( Z<0 ) Z=0; 
        break;


   case funUpperSphere:
        if( X > -105 && X < 105 && Y > -105 && Y < 105 ) {
            Z = 10000 - X*X - Y*Y;
                if( Z < 0 ) Z=0;
        }
        Z = root(Z);
        break;

   case funLowSphere:
        if( X > -105 && X < 105 && Y > -105 && Y < 105 ) {
            Z = 10000 - X*X - Y*Y;
            if( Z < 0 ) Z=0;
        }
        Z = -root(Z);
        break;
  
   case funY2mX2:
        if (Y > X && Y > -X ) { 
           Z = 1 * ((int) (Y*Y - X*X )/100);  //Y=T 
        }  else {               
           Z = 1 * ((int) (Y*Y - X*X )/100);  //Y=T 
        }
        break;

   case funFlippedParabola:
        Z = - (int) (Y*Y + X*X)/50;
        break;


   case 7: 
        Z = (int) 1000000 /(Y*Y + X*X + 50000);
        break;
    
   case funPolinomials:
        Z = (X+300) * X / 600 * (X+200) / 800 * (X-200) / 600 * 
            (Y+300) * Y / 800 * (Y+200) / 800 * (Y-200) / 1000;
        if( Z < 0 ) Z=0;
        break;
   }

   //if( addFunctionToTerrainFunction ) Z += Terrain(X+hFFx, Y+hFFy);
   return Z;

}
//---------------------------------------------------------------------------
//   F U N C T I O N S   S E L E C T E D   B Y   F U N C T.  N U M B E R
//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

       
                       



//Square root calculations: t - target, returns root.
public int root( int t) {
	int acc 	= 0;
	int x		= 1;
	int z		= 0;
	int d		= t;
	int r		= 0;
	int r2  	= r<<=1;
	int w		= t;

	while ( w>0 ) { w>>>=2; z++; x<<=1; }
	x>>>=1; z--; 
	// now x is maximum binary unit that approximate root from the bottom
	
	while ( x!=0 ) {
		w=d-((r2 + x)<<z);
	
		if( w==0 ) return ( r+x );
		if( w<0  ) { if( (--z)< acc ) return r;
				 x>>>=1; 
				 continue;
		}
		r+=x;
		r2 = (r<<1);
		if( (--z)< acc ) return r;
		x>>>=1;
		d=w;
	}
	return r;
}

/** Just a template for children classes. 
    Can be overriden. */
public void InsertSubHill(int iii, int frontDistance) {
}


}
